package de.hdm_stuttgart.battlearena.Model.Map;

import javafx.scene.canvas.GraphicsContext;
import javafx.scene.image.Image;

import org.apache.logging.log4j.Logger;
import org.apache.logging.log4j.LogManager;

import java.util.Objects;

public class TileManager {

    private static final Logger log = LogManager.getLogger(TileManager.class);

    private final GraphicsContext graphicsContext2D;
    private final ITile[] tileSet;
    private final int[][] tileMap;

    public TileManager(GraphicsContext graphicsContext2D, int diffTileCount,
                       int horizontalTileCount, int verticalTileCount, String mapString) {
        this.graphicsContext2D = graphicsContext2D;

        tileSet = new BackgroundTile[diffTileCount];
        tileMap = new int[horizontalTileCount][verticalTileCount];

        createTiles();
        generateMap(mapString, horizontalTileCount, verticalTileCount);
    }

    private void createTiles() {
        try {
            tileSet[0] = TileFactory.createTile(TileType.WALKABLE,
                    new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/map/mapPlaceholder.png"))));
            tileSet[1] = TileFactory.createTile(TileType.NON_WALKABLE,
                    new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/map/stoneExampleTexture.png"))));
        } catch (Exception e) {
            log.error(e);
        }
    }

    private void generateMap(String mapString, int horizontalTileCount, int verticalTileCount) {
        String[] tileSet = mapString.split(" ");

        int stringIndex = 0;

        for (int column = 0; column < horizontalTileCount; column++) {
            for (int row = 0; row < verticalTileCount; row++) {
                tileMap[column][row] = Integer.parseInt(tileSet[stringIndex]);
                stringIndex++;
            }
        }
    }

    public void renderMap(int horizontalTileCount, int verticalTileCount) {
        int currentTile;
        int scaledTileSize = 48; //todo: later replace "48" with value from gameScene.
        int tilePosX;
        int tilePosY;

        for (int mapColumn = 0; mapColumn < horizontalTileCount; mapColumn++) {
            for (int mapRow = 0; mapRow < verticalTileCount; mapRow++) {
                currentTile = tileMap[mapRow][mapColumn];

                tilePosX = mapColumn * scaledTileSize;
                tilePosY = mapRow * scaledTileSize;

                graphicsContext2D.drawImage(tileSet[currentTile].getTileSprite(), tilePosX, tilePosY,
                        scaledTileSize, scaledTileSize);
            }
        }
    }
}