package de.hdm_stuttgart.battlearena.Model.Multiplayer;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import java.net.*;
import java.io.*;
import java.util.concurrent.Executors;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.TimeUnit;

public class Server {

    private static final Logger log = LogManager.getLogger(Server.class);
    private ServerSocket serverSocket;
    private static boolean isServerRunning = true;
    private static int px;
    private static int py;

    private static int enemyx;
    private static int enemyy;

    private static int[] cords;

    public static void main(String[] args) throws IOException {
        Server server = new Server();
        log.info("server starting...");
        server.start("localhost",4444);
    }

    public void start(String lolcalhost, int port) throws IOException {
        serverSocket = new ServerSocket(port);
        log.info("server started!");
        new ServerHandler(serverSocket.accept()).start();
    }

    private static class ServerHandler extends Thread {
        private Socket clientSocket;
        private PrintWriter out;
        private BufferedReader in;

        public ServerHandler(Socket socket) {
            this.clientSocket = socket;
        }

        public void run() {
            ScheduledExecutorService executorService = Executors.newSingleThreadScheduledExecutor();

            executorService.scheduleAtFixedRate(() -> {
                try {
                    BufferedReader localIn = new BufferedReader(new InputStreamReader(clientSocket.getInputStream()));
                    PrintWriter localOut = new PrintWriter(clientSocket.getOutputStream(), true);

                    String inputLine = localIn.readLine();

                    log.info(inputLine);

                    if (inputLine != null) {
                        // Assuming that the input line is a comma-separated list of integers
                        String[] coordinates = inputLine.split(",");
                        cords = new int[coordinates.length];
                        for (int i = 0; i < coordinates.length; i++) {
                            cords[i] = Integer.parseInt(coordinates[i]);
                        }
                    }

                    log.info(cords);

                    //check if server was shut down:
                    if (!isServerRunning) {
                        executorService.shutdown();
                        try {
                            if (!executorService.awaitTermination(5, TimeUnit.SECONDS)) {
                                executorService.shutdownNow();
                            }
                        } catch (InterruptedException e) {
                            log.error("Error waiting for executor service termination: " + e.getMessage());
                        }
                    }

                } catch (IOException e) {
                    throw new RuntimeException(e);
                }
            }, 0, 250, TimeUnit.MILLISECONDS);
        }
    }

    public void stopServer() {
        isServerRunning = false;
        try {
            serverSocket.close(); // Close the server socket to break out of accept() in the main thread
        } catch (IOException e) {
            log.error("Error closing server socket: " + e.getMessage());
        }
    }

}
